@@ -35,4 +35,12 @@ EsfUsd_Attribute::_GetQuery() const
     };
 }
 
-PXR_NAMESPACE_CLOSE_SCOPE
\ No newline at end of file
+SdfPathVector
+EsfUsd_Attribute::_GetConnections() const
+{
+    SdfPathVector targets;
+    _GetWrapped().GetConnections(&targets);
+    return targets;
+}
+
+PXR_NAMESPACE_CLOSE_SCOPE
