@@ -17,10 +17,15 @@
 PXR_NAMESPACE_OPEN_SCOPE
 
 ExecUsdValueKey::ExecUsdValueKey(const UsdAttribute &provider)
-    : _key(ExecUsd_AttributeValueKey{
+    : _key(ExecUsd_AttributeComputationValueKey{
             provider, ExecBuiltinComputations->computeValue})
 {}
 
+ExecUsdValueKey::ExecUsdValueKey(
+    const UsdAttribute &provider, const TfToken &computation)
+    : _key(ExecUsd_AttributeComputationValueKey{provider, computation})
+{}
+
 ExecUsdValueKey::ExecUsdValueKey(
     const UsdPrim &provider, const TfToken &computation)
     : _key(ExecUsd_PrimComputationValueKey{provider, computation})
