@@ -80,6 +80,8 @@ using _BToSMap = std::unordered_map<TfToken,
 
 static TfStaticData<_SToBMap> Hd_SPrimSToBFncs;
 static TfStaticData<_BToSMap> Hd_SPrimBToSFncs;
+static TfStaticData<_SToBMap> Hd_RPrimSToBFncs;
+static TfStaticData<_BToSMap> Hd_RPrimBToSFncs;
 
 /*static*/
 void
@@ -218,6 +220,14 @@ HdDirtyBitsTranslator::RprimDirtyBitsToLocatorSet(TfToken const& primType,
     if (bits & HdChangeTracker::DirtyTransform) {
         set->append(HdXformSchema::GetDefaultLocator());
     }
+
+    if (!Hd_RPrimBToSFncs->empty()) {
+        const auto fncIt = Hd_RPrimBToSFncs->find(primType);
+        if (fncIt != Hd_RPrimBToSFncs->end()) {
+            // call custom handler registered for this type
+            fncIt->second(bits, set);
+        }
+    }
 }
 
 /* static */
@@ -492,6 +502,12 @@ _FindLocator(HdDataSourceLocator const& locator,
         return false;
     }
 
+    // Check for a universal locator set.  If we find it we never want
+    // to advance regardless of advanceToNext.
+    if ((*it)->IsEmpty()) {
+        return true;
+    }
+
     // The range between *it and end can be divided into:
     // 1.) items < locator and not a prefix.
     // 2.) items < locator and a prefix.
@@ -794,7 +810,15 @@ HdDirtyBitsTranslator::RprimLocatorSetToDirtyBits(
     if (_FindLocator(HdXformSchema::GetDefaultLocator(), end, &it)) {
         bits |= HdChangeTracker::DirtyTransform;
     }
-
+    
+    if (!Hd_RPrimSToBFncs->empty())
+    {
+        const auto fncIt = Hd_RPrimSToBFncs->find(primType);
+        if (fncIt != Hd_RPrimSToBFncs->end()) {
+            // call custom handler registered for this type
+            fncIt->second(set, &bits);
+        }
+    }
     return bits;
 }
 
@@ -845,6 +869,12 @@ HdDirtyBitsTranslator::SprimLocatorSetToDirtyBits(
                     }
                     else if (terminal == HdMaterialSchemaTokens->volume) {
                         bits |= HdMaterial::DirtyVolume;
+                    } else {
+                        // There is no specific terminal mentioned,
+                        // so consider the entire material dirty.
+                        // This can happen, for example, when a
+                        // material/{renderContext} locator is invalidated.
+                        bits |= HdMaterial::AllDirty;
                     }
                 }
             }
@@ -1201,4 +1231,14 @@ HdDirtyBitsTranslator::RegisterTranslatorsForCustomSprimType(
     Hd_SPrimBToSFncs->insert({primType, bToSFnc});
 }
 
+void
+HdDirtyBitsTranslator::RegisterTranslatorsForCustomRprimType(
+    TfToken const& primType,
+    LocatorSetToDirtyBitsFnc sToBFnc,
+    DirtyBitsToLocatorSetFnc bToSFnc)
+{
+    Hd_RPrimSToBFncs->insert({primType, sToBFnc});
+    Hd_RPrimBToSFncs->insert({primType, bToSFnc});
+}
+
 PXR_NAMESPACE_CLOSE_SCOPE
