@@ -22,6 +22,13 @@ HdRendererPluginHandle::HdRendererPluginHandle(
     }
 }
 
+HdRendererPluginHandle::HdRendererPluginHandle(
+    HdRendererPluginHandle &&other)
+  : _plugin(other._plugin)
+{
+    other._plugin = nullptr;
+}
+
 HdRendererPluginHandle::~HdRendererPluginHandle()
 {
     HdRendererPluginRegistry::GetInstance().ReleasePlugin(_plugin);
@@ -30,11 +37,25 @@ HdRendererPluginHandle::~HdRendererPluginHandle()
 HdRendererPluginHandle &
 HdRendererPluginHandle::operator=(const HdRendererPluginHandle &other)
 {
+    // Increase reference count first to do the right thing for myHandle = myHandle.
+    if (other._plugin) {
+        HdRendererPluginRegistry::GetInstance().AddPluginReference(other._plugin);
+    }
     HdRendererPluginRegistry::GetInstance().ReleasePlugin(_plugin);
     _plugin = other._plugin;
-    if (_plugin) {
-        HdRendererPluginRegistry::GetInstance().AddPluginReference(_plugin);
+    return *this;
+}
+
+HdRendererPluginHandle &
+HdRendererPluginHandle::operator=(HdRendererPluginHandle &&other)
+{
+    if (this == &other) {
+        return *this;
     }
+
+    HdRendererPluginRegistry::GetInstance().ReleasePlugin(_plugin);
+    _plugin = other._plugin;
+    other._plugin = nullptr;
     return *this;
 }
 
