@@ -26,6 +26,9 @@ HgiGLComputePipeline::~HgiGLComputePipeline() = default;
 void
 HgiGLComputePipeline::BindPipeline()
 {
+    // Default to seamless cubemap sampling.
+    glEnable(GL_TEXTURE_CUBE_MAP_SEAMLESS);
+
     //
     // Shader program
     //
