@@ -38,8 +38,6 @@ using std::vector;
 
 PXR_NAMESPACE_OPEN_SCOPE
 
-SDF_DEFINE_SPEC(SdfSchema, SdfSpecTypePrim, SdfPrimSpec, SdfSpec);
-
 // register types
 TF_REGISTRY_FUNCTION(TfType)
 {
@@ -507,10 +505,10 @@ SdfPrimSpec::GetRelationshipAtPath(const SdfPath& path) const
 #define SDF_ACCESSOR_WRITE_PREDICATE(key_)   _ValidateEdit(key_)
 
 SDF_DEFINE_GET(TypeName, SdfFieldKeys->TypeName, TfToken)
+SDF_DEFINE_GET(Hidden,   SdfFieldKeys->Hidden,   bool)
 
 SDF_DEFINE_GET_SET(Comment,            SdfFieldKeys->Comment,       std::string)
 SDF_DEFINE_GET_SET(Documentation,      SdfFieldKeys->Documentation, std::string)
-SDF_DEFINE_GET_SET(Hidden,             SdfFieldKeys->Hidden,        bool)
 SDF_DEFINE_GET_SET(SymmetryFunction,   SdfFieldKeys->SymmetryFunction, TfToken)
 SDF_DEFINE_GET_SET(SymmetricPeer,      SdfFieldKeys->SymmetricPeer, std::string)
 SDF_DEFINE_GET_SET(Prefix,             SdfFieldKeys->Prefix,        std::string)
@@ -533,6 +531,9 @@ SDF_DEFINE_TYPED_GET_SET(Permission, SdfFieldKeys->Permission,
 SDF_DEFINE_DICTIONARY_GET_SET(GetSymmetryArguments,
                               SetSymmetryArgument, 
                               SdfFieldKeys->SymmetryArguments);
+SDF_DEFINE_DICTIONARY_GET_SET(GetClips,
+                              SetClips,
+                              SdfFieldKeys->Clips);
 SDF_DEFINE_DICTIONARY_GET_SET(GetCustomData,
                               SetCustomData,
                               SdfFieldKeys->CustomData);
@@ -558,6 +559,16 @@ SdfPrimSpec::SetTypeName(const std::string& value)
     }
 }
 
+void
+SdfPrimSpec::SetHidden(bool value)
+{
+    if (TfGetEnvSetting(SDF_LEGACY_UI_HINTS_WARN_ON_WRITE)) {
+        TF_WARN("Writing to deprecated metadata field 'hidden'");
+    }
+
+    SetField(SdfFieldKeys->Hidden, value);
+}
+
 //
 // Inherits
 //
@@ -665,9 +676,8 @@ SdfPrimSpec::ClearReferenceList()
 SdfVariantSetNamesProxy
 SdfPrimSpec::GetVariantSetNameList() const
 {
-    return SdfVariantSetNamesProxy(
-        std::make_unique<Sdf_ListOpListEditor<SdfNameKeyPolicy>>(
-            SdfCreateHandle(this), SdfFieldKeys->VariantSetNames));
+    return SdfGetNameEditorProxy(
+            SdfCreateHandle(this), SdfFieldKeys->VariantSetNames);
 }
 
 bool
@@ -793,6 +803,22 @@ SdfPrimSpec::ClearRelocates()
     }
 }
 
+//
+// ClipSets
+//
+SdfNameEditorProxy
+SdfPrimSpec::GetClipSetsList() const
+{
+    return SdfGetNameEditorProxy(
+        SdfCreateHandle(this), SdfFieldKeys->ClipSets);
+}
+
+bool
+SdfPrimSpec::HasClipSets() const
+{
+    return GetClipSetsList().HasKeys();
+}
+
 //
 // Utilities
 //
