@@ -11,6 +11,7 @@
 
 #include "pxr/imaging/hd/basisCurvesSchema.h"
 #include "pxr/imaging/hd/basisCurvesTopologySchema.h"
+#include "pxr/imaging/hd/builtinMaterialSchema.h"
 #include "pxr/imaging/hd/dataSource.h"
 #include "pxr/imaging/hd/dataSourceTypeDefs.h"
 #include "pxr/imaging/hd/extentSchema.h"
@@ -25,6 +26,7 @@
 #include "pxr/imaging/hd/materialSchema.h"
 #include "pxr/imaging/hd/meshSchema.h"
 #include "pxr/imaging/hd/meshTopologySchema.h"
+#include "pxr/imaging/hd/primOriginSchema.h"
 #include "pxr/imaging/hd/primvarSchema.h"
 #include "pxr/imaging/hd/primvarsSchema.h"
 #include "pxr/imaging/hd/purposeSchema.h"
@@ -61,8 +63,7 @@ HdSceneIndexPrim
 UsdImaging_DrawModeStandin::GetPrim(const SdfPath& path) const
 {
     const SdfPath relPath = path.MakeRelativePath(_path);
-    return { _GetPrimType(relPath), _GetPrimSource(relPath)
-    };
+    return { _GetPrimType(relPath), _GetPrimSource(relPath) };
 }
 
 SdfPathVector
@@ -352,6 +353,7 @@ protected:
 /// - purpose (from the given prim data source)
 /// - visibility (from the given prim data source)
 /// - displayStyle (constant)
+/// - primOrigin (HdPrimOriginSchema to map picking back to the USD prim)
 ///
 class _PrimDataSource : public HdContainerDataSource
 {
@@ -363,14 +365,16 @@ public:
             HdPurposeSchemaTokens->purpose,
             HdVisibilitySchemaTokens->visibility,
             HdInstancedBySchemaTokens->instancedBy,
-            HdLegacyDisplayStyleSchemaTokens->displayStyle };
+            HdLegacyDisplayStyleSchemaTokens->displayStyle,
+            HdPrimOriginSchemaTokens->primOrigin };
     }
 
     HdDataSourceBaseHandle Get(const TfToken &name) override {
         if (name == HdXformSchemaTokens->xform ||
             name == HdPurposeSchemaTokens->purpose ||
             name == HdVisibilitySchemaTokens->visibility ||
-            name == HdInstancedBySchemaTokens->instancedBy) {
+            name == HdInstancedBySchemaTokens->instancedBy ||
+            name == HdPrimOriginSchemaTokens->primOrigin) {
             if (_primSource) {
                 return _primSource->Get(name);
             }
@@ -382,6 +386,8 @@ public:
                     .SetCullStyle(
                         HdRetainedTypedSampledDataSource<TfToken>::New(
                             HdCullStyleTokens->back))
+                    .SetMaterialIsFinal(
+                        HdRetainedTypedSampledDataSource<bool>::New(true))
                     .Build();
             return src;
         }
@@ -1063,7 +1069,6 @@ private:
     /// The cached data.
     struct _CardsData
     {
-
         _CardsData(const _SchemaValues &values, const SdfPath &primPath);
 
         TfToken cardGeometry;
@@ -1088,7 +1093,8 @@ private:
         _ComputeUVs(const _SchemaValues &values);
         static
         _NameToContainer
-        _ComputeGeomSubsets(const _SchemaValues &values,
+        _ComputeGeomSubsets(
+            const _SchemaValues &values,
             const SdfPath &primPath);
         static const
         _NameToContainer
@@ -1436,6 +1442,12 @@ _CardsDataCache::_CardsData::_ComputeGeomSubsets(
 
     _NameToContainer subsets;
 
+    static const HdDataSourceBaseHandle materialIsFinalDs =
+        HdLegacyDisplayStyleSchema::Builder()
+            .SetMaterialIsFinal(
+                HdRetainedTypedSampledDataSource<bool>::New(true))
+            .Build();
+
     // Do not generate subsets if there are no textures for any face.
     // The entire standin prim will use the renderer's fallback material, which
     // should pick up displayColor and displayOpacity.
@@ -1507,7 +1519,11 @@ _CardsDataCache::_CardsData::_ComputeGeomSubsets(
                     HdMaterialBindingsSchema::BuildRetained(
                         TfArraySize(materialBindingNames),
                         materialBindingNames,
-                        materialBindingSources)) });
+                        materialBindingSources),
+
+                    HdLegacyDisplayStyleSchema::GetSchemaToken(),
+                    materialIsFinalDs) 
+            });
         }
     }
 
@@ -1769,6 +1785,12 @@ _CardsDataCache::_CardsData::_ComputeMaterials(const _SchemaValues &values)
 
     _NameToContainer materials;
 
+    static HdDataSourceBaseHandle const builtinMaterialDs =
+    HdBuiltinMaterialSchema::Builder()
+        .SetBuiltinMaterial(
+            HdRetainedTypedSampledDataSource<bool>::New(true))
+        .Build();
+
     // do not generate any materials if there are no textures for any face
     if (values.hasTexture.count()) {
         for (auto i = 0; i < 6; ++i) {
@@ -1823,7 +1845,10 @@ _CardsDataCache::_CardsData::_ComputeMaterials(const _SchemaValues &values)
                         HdMaterialSchema::BuildRetained(
                             networkNames.size(),
                             networkNames.data(),
-                            networks.data())));
+                            networks.data()),
+                        HdBuiltinMaterialSchema::GetSchemaToken(),
+                        builtinMaterialDs
+                    ));
             }
         }
     }
