@@ -23,7 +23,7 @@
 #include "pxr/imaging/hd/overlayContainerDataSource.h"
 #include "pxr/imaging/hd/retainedDataSource.h"
 #include "pxr/imaging/hd/sceneIndex.h"
-#include "pxr/imaging/hd/xformSchema.h"
+#include "pxr/imaging/hd/tokens.h"
 
 #include "pxr/base/trace/trace.h"
 
@@ -180,6 +180,7 @@ UsdSkelImagingDataSourceResolvedPointsBasedPrim(
  , _resolvedSkeletonSchema(std::move(resolvedSkeletonSchema))
  , _blendShapeDataCache(sceneIndex, primPath)
  , _jointInfluencesDataCache(_primSource, _skeletonPrimSource)
+ , _xformResolver(sceneIndex, _primSource)
 {
 }
 
@@ -221,12 +222,14 @@ _ExtComputationPrimvars(const SdfPath &primPath)
 
 static
 HdContainerDataSourceHandle
-_BlockPointsPrimvars()
+_BlockPointsAndNormalsPrimvars()
 {
     const TfToken names[] = {
-        HdPrimvarsSchemaTokens->points
+        HdPrimvarsSchemaTokens->points,
+        HdPrimvarsSchemaTokens->normals
     };
     HdDataSourceBaseHandle const values[] = {
+        HdBlockDataSource::New(),
         HdBlockDataSource::New()
     };
 
@@ -251,7 +254,9 @@ UsdSkelImagingDataSourceResolvedPointsBasedPrim::Get(const TfToken &name)
 
     if (name == HdPrimvarsSchema::GetSchemaToken()) {
         // Block points primvar.
-        static HdContainerDataSourceHandle ds = _BlockPointsPrimvars();
+        // The normals are also blocked so they are recomputed after skinning,
+        // since normals currently aren't deformed by the computation.
+        static HdContainerDataSourceHandle ds = _BlockPointsAndNormalsPrimvars();
         return ds;
     }
 
@@ -282,10 +287,10 @@ UsdSkelImagingDataSourceResolvedPointsBasedPrim::GetNames()
 }
 
 HdMatrixDataSourceHandle
-UsdSkelImagingDataSourceResolvedPointsBasedPrim::GetPrimWorldToLocal() const {
+UsdSkelImagingDataSourceResolvedPointsBasedPrim::
+GetCommonSpaceToPrimLocal() const {
     return
-        _MatrixInverseDataSource::New(
-            HdXformSchema::GetFromParent(_primSource).GetMatrix());
+        _MatrixInverseDataSource::New(_xformResolver.GetPrimLocalToCommonSpace());
 }
 
 std::shared_ptr<UsdSkelImagingBlendShapeData>
@@ -306,7 +311,7 @@ UsdSkelImagingDataSourceResolvedPointsBasedPrim::GetDependendendOnDataSourceLoca
     static const HdDataSourceLocatorSet result{
         UsdSkelImagingBindingSchema::GetDefaultLocator(),
         HdPrimvarsSchema::GetDefaultLocator(),
-        HdXformSchema::GetDefaultLocator()
+        UsdSkelImagingDataSourceXformResolver::GetXformLocator()
     };
 
     return result;
@@ -318,6 +323,8 @@ UsdSkelImagingDataSourceResolvedPointsBasedPrim::_ProcessDirtyLocators(
     HdDataSourceLocatorSet * const dirtyLocatorsForAggregatorComputation,
     HdDataSourceLocatorSet * const dirtyLocatorsForComputation)
 {
+    TRACE_FUNCTION();
+    
     if (dirtyLocators.Contains(
             UsdSkelImagingBindingSchema::GetSkeletonLocator())) {
         return true;
@@ -334,6 +341,12 @@ UsdSkelImagingDataSourceResolvedPointsBasedPrim::_ProcessDirtyLocators(
             HdPrimvarsSchema::GetDefaultLocator())) {
         return true;
     }
+    if (dirtyLocators.Contains(
+            UsdSkelImagingDataSourceXformResolver::GetInstancedByLocator())) {
+        // Instancers have changed.
+        // Just indicate that we want to blow everything.
+        return true;
+    }
     static const HdDataSourceLocator skinningMethodLocator =
         HdPrimvarsSchema::GetDefaultLocator()
             .Append(UsdSkelImagingBindingSchemaTokens->skinningMethodPrimvar);
@@ -446,13 +459,14 @@ UsdSkelImagingDataSourceResolvedPointsBasedPrim::_ProcessDirtyLocators(
         }
     }
 
-    if (dirtyLocators.Intersects(HdXformSchema::GetDefaultLocator())) {
+    if (dirtyLocators.Intersects(
+            UsdSkelImagingDataSourceXformResolver::GetXformLocator())) {
         if (dirtyLocatorsForComputation) {
             static const HdDataSourceLocator inputLocator =
                 HdExtComputationSchema::GetInputValuesLocator()
                     .Append(
                         UsdSkelImagingExtComputationInputNameTokens
-                        ->primWorldToLocal);
+                        ->commonSpaceToPrimLocal);
             dirtyLocatorsForComputation->insert(inputLocator);
         }
     }
@@ -466,7 +480,8 @@ UsdSkelImagingDataSourceResolvedPointsBasedPrim::_ProcessDirtySkeletonLocators(
     HdDataSourceLocatorSet * const dirtyLocatorsForAggregatorComputation,
     HdDataSourceLocatorSet * const dirtyLocatorsForComputation)
 {
-
+    TRACE_FUNCTION();
+    
     if (dirtyLocators.Contains(
             UsdSkelImagingResolvedSkeletonSchema::GetDefaultLocator())) {
         return true;
@@ -478,13 +493,13 @@ UsdSkelImagingDataSourceResolvedPointsBasedPrim::_ProcessDirtySkeletonLocators(
 
     if (dirtyLocators.Intersects(
             UsdSkelImagingResolvedSkeletonSchema::
-            GetSkelLocalToWorldLocator())) {
+            GetSkelLocalToCommonSpaceLocator())) {
         if (dirtyLocatorsForComputation) {
             static const HdDataSourceLocator inputLocator =
                 HdExtComputationSchema::GetInputValuesLocator()
                     .Append(
                         UsdSkelImagingExtComputationInputNameTokens
-                        ->skelLocalToWorld);
+                        ->skelLocalToCommonSpace);
             dirtyLocatorsForComputation->insert(
                 inputLocator);
         }
@@ -528,6 +543,8 @@ UsdSkelImagingDataSourceResolvedPointsBasedPrim::_ProcessDirtySkelBlendShapeLoca
         HdDataSourceLocatorSet * const dirtyLocatorsForAggregatorComputation,
         HdDataSourceLocatorSet * const dirtyLocatorsForComputation)
 {
+    TRACE_FUNCTION();
+
     if (!_resolvedSkeletonSchema) {
         return false;
     }
@@ -568,12 +585,48 @@ UsdSkelImagingDataSourceResolvedPointsBasedPrim::_ProcessDirtySkelBlendShapeLoca
     return false;
 }
 
+bool
+UsdSkelImagingDataSourceResolvedPointsBasedPrim::_ProcessDirtyInstancerLocators(
+        const HdDataSourceLocatorSet &dirtyLocators,
+        HdDataSourceLocatorSet * const dirtyLocatorsForAggregatorComputation,
+        HdDataSourceLocatorSet * const dirtyLocatorsForComputation)
+{
+    TRACE_FUNCTION();
+
+    if (dirtyLocators.Intersects(
+            UsdSkelImagingDataSourceXformResolver::GetInstancedByLocator())) {
+        return true;
+    }
+
+    if (!_resolvedSkeletonSchema) {
+        return false;
+    }
+
+    if (dirtyLocators.Intersects(
+            UsdSkelImagingDataSourceXformResolver::GetXformLocator()) ||
+        dirtyLocators.Intersects(
+            UsdSkelImagingDataSourceXformResolver::GetInstanceXformLocator())) {
+        if (dirtyLocatorsForComputation) {
+            static const HdDataSourceLocator inputLocator =
+                HdExtComputationSchema::GetInputValuesLocator()
+                    .Append(
+                        UsdSkelImagingExtComputationInputNameTokens
+                        ->commonSpaceToPrimLocal);
+            dirtyLocatorsForComputation->insert(inputLocator);
+        }
+    }
+
+    return false;
+}
+
 bool
 UsdSkelImagingDataSourceResolvedPointsBasedPrim::ProcessDirtyLocators(
     const TfToken &dirtiedPrimType,
     const HdDataSourceLocatorSet &dirtyLocators,
     HdSceneIndexObserver::DirtiedPrimEntries * const entries)
 {
+    TRACE_FUNCTION();
+    
     HdDataSourceLocatorSet dirtyLocatorsForAggregatorComputation;
     HdDataSourceLocatorSet dirtyLocatorsForComputation;
 
@@ -596,6 +649,11 @@ UsdSkelImagingDataSourceResolvedPointsBasedPrim::ProcessDirtyLocators(
             dirtyLocators,
             dirtyLocatorsForAggregatorComputationPtr,
             dirtyLocatorsForComputationPtr);
+    } else if (dirtiedPrimType == HdPrimTypeTokens->instancer) {
+        result = _ProcessDirtyInstancerLocators(
+            dirtyLocators,
+            dirtyLocatorsForAggregatorComputationPtr,
+            dirtyLocatorsForComputationPtr);
     } else {
         result = _ProcessDirtyLocators(
             dirtyLocators,
