@@ -15,6 +15,7 @@
 #include "pxr/usd/usd/editContext.h"
 #include "pxr/usd/usd/variantSets.h"
 #include "pxr/usdValidation/usdValidation/context.h"
+#include "pxr/usdValidation/usdValidation/fixer.h"
 #include "pxr/usdValidation/usdValidation/registry.h"
 #include "pxr/usdValidation/usdValidation/validatorTokens.h"
 #include "pxr/usdValidation/usdUtilsValidators/validatorTokens.h"
@@ -61,7 +62,7 @@ struct Args {
     bool dumpRules = false;
     bool verbose = false;
     bool strict = false;
-    bool useNewValidationFramework = false;
+    bool useOldComplianceCheckerInterface = false;
 };
 
 static
@@ -108,8 +109,13 @@ _Configure(CLI::App* app, Args& args) {
         "Return failure code even if only warnings are issued, for stricter\n"
         "compliance.");
     app->add_flag(
-        "--useNewValidationFramework", args.useNewValidationFramework, 
-        "Enable the new validation framework.");
+        "--useNewValidationFramework",
+        "Default behavior, this option is IGNORED but retained to avoid client "
+        "code breakage.")->group("");
+    app->add_flag(
+        "--useOldComplianceCheckerInterface", 
+        args.useOldComplianceCheckerInterface, 
+        "Use the old and now deprecated Compliance Checker interface.");
     app->add_option(
         "--variantSets", args.variantSets,
         "List of variantSets to validate. All variants for the given\n"
@@ -153,15 +159,15 @@ _ValidateArgs(const Args& args) {
     }
 
     // variants option is only valid when using new validation framework.
-    if (!args.variants.empty() && !args.useNewValidationFramework) {
+    if (!args.variants.empty() && args.useOldComplianceCheckerInterface) {
         std::cerr<<"Error: The --variants option is only valid when using the "
-            "--useNewValidationFramework option."<<"\n";
+            "new ValidationFramework."<<"\n";
         return false;
     }
 
-    if (!args.variantSets.empty() && !args.useNewValidationFramework) {
+    if (!args.variantSets.empty() && args.useOldComplianceCheckerInterface) {
         std::cerr<<"Error: The --variantSets option is only valid when using "
-            "the --useNewValidationFramework option."<<"\n";
+            "the new ValidationFramework."<<"\n";
         return false;
     }
 
@@ -217,6 +223,25 @@ _ReportValidationErrors(
             case UsdValidationErrorType::Error:
                 reportFailure = true;
                 _PrintMessage(output, error.GetErrorAsString(), _ErrorColor);
+                if (!error.GetFixers().empty()) {
+                    _PrintMessage(
+                        output, 
+                        "\tPossible Fixes which can be applied:", _InfoColor);
+                    for (const UsdValidationFixer *fixer : error.GetFixers()) {
+                        if (fixer->CanApplyFix(
+                                error, UsdEditTarget(
+                                error.GetSites()[0].GetStage()->GetRootLayer()),
+                                UsdTimeCode::Default())) {
+                            _PrintMessage(
+                                output, 
+                                TfStringPrintf(
+                                    "\t- %s: %s.", 
+                                    fixer->GetName().GetText(), 
+                                    fixer->GetDescription().c_str()),
+                                _InfoColor);
+                        }
+                    }
+                }
                 break;
             case UsdValidationErrorType::Warn:
                 if (strict) {
@@ -540,7 +565,7 @@ _UsdChecker(const Args& args)
         return 1;
     }
 
-    if (args.useNewValidationFramework) {
+    if (!args.useOldComplianceCheckerInterface) {
         UsdValidationRegistry &validationReg = 
             UsdValidationRegistry::GetInstance();
         UsdValidationValidatorMetadataVector metadata = 
@@ -671,7 +696,7 @@ _UsdChecker(const Args& args)
 
     std::cerr<<"usdchecker using UsdUtilsComplianceChecker requires Python "
         "support to be enabled in the build of USD. Its recommended to use "
-        "--useNewValidationFramework which doesn't require any python "
+        "the new ValidationFramework which doesn't require any python "
         "support.\n";
     return 1;
 
@@ -786,8 +811,8 @@ main(int argc, char const *argv[]) {
         "attribute is checked, currently.  General USD checks are always "
         "performed, and more restrictive checks targeted at distributable "
         "consumer content are also applied when the \"--arkit\" option is "
-        "specified. In order to use the new validation framework provide the "
-        "'--useNewValidationFramework' option.");
+        "specified. In order to use the old compliance checker (deprecated) "
+        "provide the '--useOldComplianceCheckerInterface' option.");
 
     Args args;
     _Configure(&app, args);
