@@ -10,6 +10,7 @@
 #include "pxr/usd/ar/resolver.h"
 #include "pxr/usd/sdr/registry.h"
 #include "pxr/usd/sdr/shaderProperty.h"
+#include "pxr/usd/usd/editTarget.h"
 #include "pxr/usd/usd/prim.h"
 #include "pxr/usd/usd/property.h"
 #include "pxr/usd/usd/relationship.h"
@@ -23,6 +24,7 @@
 #include "pxr/usd/usdShade/tokens.h"
 #include "pxr/usdValidation/usdShadeValidators/validatorTokens.h"
 #include "pxr/usdValidation/usdValidation/error.h"
+#include "pxr/usdValidation/usdValidation/fixer.h"
 #include "pxr/usdValidation/usdValidation/registry.h"
 #include "pxr/usdValidation/usdValidation/timeRange.h"
 
@@ -667,12 +669,14 @@ _NormalMapTextureValidator(const UsdPrim& usdPrim,
             UsdShadeValidationErrorNameTokens->invalidFile,
             UsdValidationErrorType::Error,
             UsdValidationErrorSites{
-                    UsdValidationErrorSite(usdPrim.GetStage(),
-                                           sourcePrim.GetPath())
+                UsdValidationErrorSite(usdPrim.GetStage(), usdPrim.GetPath()),
+                UsdValidationErrorSite(usdPrim.GetStage(), sourcePrim.GetPath())
             },
-            TfStringPrintf("UsdUVTexture prim <%s> has invalid or "
+            TfStringPrintf("UsdUVTexture prim <%s> (connected through "
+                           "validating prim <%s>) has invalid or "
                            "unresolvable inputs:file of @%s@",
-                           sourcePrim.GetPath().GetText(), assetPath.c_str()));
+                           sourcePrim.GetPath().GetText(), 
+                           usdPrim.GetPath().GetText(), assetPath.c_str()));
     }
 
     auto _TextureIs8Bit = [](std::string resolvedPath) {
@@ -700,13 +704,14 @@ _NormalMapTextureValidator(const UsdPrim& usdPrim,
             UsdShadeValidationErrorNameTokens->invalidSourceColorSpace,
             UsdValidationErrorType::Error,
             UsdValidationErrorSites{
-                    UsdValidationErrorSite(usdPrim.GetStage(),
-                                           sourcePrim.GetPath())
+                UsdValidationErrorSite(usdPrim.GetStage(), usdPrim.GetPath()),
+                UsdValidationErrorSite(usdPrim.GetStage(), sourcePrim.GetPath())
             },
-            TfStringPrintf("UsdUVTexture prim <%s> that reads"
-                           " Normal Map @%s@ should set "
-                           "inputs:sourceColorSpace to 'raw'.",
+            TfStringPrintf("UsdUVTexture prim <%s> (connected through "
+                           "validating prim <%s>) that reads Normal Map @%s@ "
+                           "should set inputs:sourceColorSpace to 'raw'.",
                            sourcePrim.GetPath().GetText(),
+                           usdPrim.GetPath().GetText(),
                            textureAssetPath.GetAssetPath().c_str()));
     }
 
@@ -726,15 +731,17 @@ _NormalMapTextureValidator(const UsdPrim& usdPrim,
             UsdShadeValidationErrorNameTokens->nonCompliantBiasAndScale,
             UsdValidationErrorType::Error,
             UsdValidationErrorSites{
-                UsdValidationErrorSite(usdPrim.GetStage(),
-                    sourcePrim.GetPath())
+                UsdValidationErrorSite(usdPrim.GetStage(), usdPrim.GetPath()),
+                UsdValidationErrorSite(usdPrim.GetStage(), sourcePrim.GetPath())
             },
-            TfStringPrintf("UsdUVTexture prim <%s> reads 8 bit Normal Map "
+            TfStringPrintf("UsdUVTexture prim <%s> (connected through "
+                           "validating prim <%s>) reads 8 bit Normal Map "
                            "@%s@, which requires that inputs:scale be set to "
                            "(2, 2, 2, 1) and inputs:bias be set to "
                            "(-1, -1, -1, 0) for proper interpretation as per "
                            "the UsdPreviewSurface and UsdUVTexture docs.",
                            sourcePrim.GetPath().GetText(),
+                           usdPrim.GetPath().GetText(),
                            textureAssetPath.GetAssetPath().c_str())
         );
         return errors;
@@ -753,10 +760,11 @@ _NormalMapTextureValidator(const UsdPrim& usdPrim,
             UsdShadeValidationErrorNameTokens->nonCompliantScale,
             UsdValidationErrorType::Warn,
             UsdValidationErrorSites{
-                UsdValidationErrorSite(usdPrim.GetStage(),
-                    sourcePrim.GetPath())
+                UsdValidationErrorSite(usdPrim.GetStage(), usdPrim.GetPath()),
+                UsdValidationErrorSite(usdPrim.GetStage(), sourcePrim.GetPath())
             },
-            TfStringPrintf("UsdUVTexture prim <%s> reads an 8 bit Normal "
+            TfStringPrintf("UsdUVTexture prim <%s> (connected through "
+                           "validating prim <%s>) reads an 8 bit Normal "
                            "Map, but has non-standard inputs:scale value "
                            "of (%.6g, %.6g, %.6g, %.6g). inputs:scale must "
                            "be set to (2, 2, 2, 1) so as fulfill the "
@@ -764,6 +772,7 @@ _NormalMapTextureValidator(const UsdPrim& usdPrim,
                            "space of [(-1,-1,-1), (1,1,1)] as documented in "
                            "the UsdPreviewSurface and UsdUVTexture docs.",
                            sourcePrim.GetPath().GetText(),
+                           usdPrim.GetPath().GetText(),
                            scaleVector[0], scaleVector[1], scaleVector[2],
                            scaleVector[3])
         );
@@ -782,32 +791,86 @@ _NormalMapTextureValidator(const UsdPrim& usdPrim,
             UsdShadeValidationErrorNameTokens->nonCompliantBias,
             UsdValidationErrorType::Warn,
             UsdValidationErrorSites{
-                UsdValidationErrorSite(usdPrim.GetStage(),
-                    sourcePrim.GetPath())
+                UsdValidationErrorSite(usdPrim.GetStage(), usdPrim.GetPath()),
+                UsdValidationErrorSite(usdPrim.GetStage(), sourcePrim.GetPath())
             },
-            TfStringPrintf("UsdUVTexture prim <%s> reads an 8 bit Normal "
-                            "Map, but has non-standard inputs:bias value of "
-                            "(%.6g, %.6g, %.6g, %.6g). inputs:bias must be "
-                            "set to [-1,-1,-1,0] so as to fulfill the "
-                            "requirements of the normals to be in tangent "
-                            "space of [(-1,-1,-1), (1,1,1)] as documented in "
-                            "the UsdPreviewSurface and UsdUVTexture docs.",
-                            sourcePrim.GetPath().GetText(),
-                            biasVector[0], biasVector[1], biasVector[2],
-                            biasVector[3])
+            TfStringPrintf("UsdUVTexture prim <%s> (connected through "
+                           "validating prim <%s>) reads an 8 bit Normal "
+                           "Map, but has non-standard inputs:bias value of "
+                           "(%.6g, %.6g, %.6g, %.6g). inputs:bias must be "
+                           "set to [-1,-1,-1,0] so as to fulfill the "
+                           "requirements of the normals to be in tangent "
+                           "space of [(-1,-1,-1), (1,1,1)] as documented in "
+                           "the UsdPreviewSurface and UsdUVTexture docs.",
+                           sourcePrim.GetPath().GetText(),
+                           usdPrim.GetPath().GetText(),
+                           biasVector[0], biasVector[1], biasVector[2],
+                           biasVector[3])
         );
     }
 
     return errors;
 }
 
+const std::vector<UsdValidationFixer>
+_MaterialBindingApiAppliedValidatorFixers() {
+    std::vector<UsdValidationFixer> fixers;
+
+    FixerCanApplyFn fixerCanApplyFn = 
+        [](const UsdValidationError &error, const UsdEditTarget &editTarget,
+           const UsdTimeCode &/*timeCode*/) -> bool {
+            if (!editTarget.IsValid() || !editTarget.GetLayer()) {
+                return false;
+            }
+            if (error.GetSites().size() != 1) {
+                // Must have one and only one error site to fix
+                return false;
+            }
+            const UsdValidationErrorSite &site = error.GetSites()[0];
+            if (!site.IsValid() || !site.IsPrim()) {
+                return false;
+            }
+            UsdPrim prim = site.GetPrim();
+            return UsdShadeMaterialBindingAPI::CanApply(prim);
+        };
+
+    FixerImplFn fixerImplFn = 
+        [](const UsdValidationError &error, const UsdEditTarget &editTarget,
+           const UsdTimeCode &/*timeCode*/) -> bool {
+            if (!editTarget.IsValid() || !editTarget.GetLayer()) {
+                return false;
+            }
+            if (error.GetSites().size() != 1) {
+                // Must have one and only one error site to fix
+                return false;
+            }
+            const UsdValidationErrorSite &site = error.GetSites()[0];
+            if (!site.IsValid() || !site.IsPrim()) {
+                return false;
+            }
+            UsdPrim prim = site.GetPrim();
+            UsdShadeMaterialBindingAPI::Apply(prim);
+            return true;
+        };
+
+    fixers.emplace_back(
+        TfToken("ApplyMaterialBindingAPI"),
+        "Applies the MaterialBindingAPI to the prim.",
+        fixerImplFn, fixerCanApplyFn, TfTokenVector{}, 
+        UsdShadeValidationErrorNameTokens
+            ->missingMaterialBindingAPI);
+
+    return fixers;
+}
+
 TF_REGISTRY_FUNCTION(UsdValidationRegistry)
 {
     UsdValidationRegistry &registry = UsdValidationRegistry::GetInstance();
 
     registry.RegisterPluginValidator(
         UsdShadeValidatorNameTokens->materialBindingApiAppliedValidator,
-        _MaterialBindingApiAppliedValidator);
+        _MaterialBindingApiAppliedValidator,
+        _MaterialBindingApiAppliedValidatorFixers());
 
     registry.RegisterPluginValidator(
         UsdShadeValidatorNameTokens->materialBindingRelationships,
